#ifndef VMS
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#else
#include <decw$include/Xlib.h>
#include <decw$include/Xutil.h>
#include <decw$include/Xos.h>
#endif

#define MINVAL(a,b)  ((a <= b) ? a : b)

#define FALSE  0
#define TRUE   1

#define MAX_COLORS  16

#define MAX_INTENSITY  65535.0

#define NUM_FONT  6
static struct { char   *name;
                float  scale; }  FONT_REF[NUM_FONT] =
       { "-Adobe-Courier-Medium-R-Normal--*-240-*-*-M-*", 19.0,
         "-Adobe-Courier-Medium-R-Normal--*-180-*-*-M-*", 13.0,
         "-Adobe-Courier-Medium-R-Normal--*-140-*-*-M-*", 10.5,
         "-Adobe-Courier-Medium-R-Normal--*-120-*-*-M-*",  8.5,
         "-Adobe-Courier-Medium-R-Normal--*-100-*-*-M-*",  7.0,
         "-Adobe-Courier-Medium-R-Normal--*-80-*-*-M-*" ,  0.0  };

static Colormap       colormap;
static Display        *dsp;
static Font           font[NUM_FONT];
static GC             obj_gc;
static Screen         *screen;
static Visual         *visual;
static Window         plot_window = 0;
static Window         root_window;
XFontStruct           *font_struc;

static int            depth;
static int            font_id;
static int            frame_no = 0;
static int            num_color;
static int            visual_class;
static unsigned int   plot_win_height;
static unsigned int   plot_win_width;
static unsigned long  BLACK;
static unsigned long  WHITE;
static unsigned long  color_table[MAX_COLORS];

static float  BLUE[MAX_COLORS] = 
              { 0.00, 1.00, 0.66, 1.00, 1.00, 1.00, 1.00, 0.80,
                0.55, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00  };
static float  GREEN[MAX_COLORS] = 
              { 0.00, 1.00, 0.66, 0.00, 0.60, 0.80, 0.90, 1.00,
                1.00, 1.00, 1.00, 1.00, 0.85, 0.70, 0.50, 0.00  };
static float  RED[MAX_COLORS] = 
              { 0.00, 1.00, 0.66, 0.00, 0.00, 0.00, 0.00, 0.00,
                0.00, 0.00, 0.70, 1.00, 1.00, 1.00, 1.00, 1.00  };

/**--------------------------------------------------------------------------**/

x11opn_(flag_success,screen_width,screen_height)
  int  *flag_success;
  int  *screen_width;
  int  *screen_height;
{

    XColor  screen_color;

    char            flag_modify = DoRed | DoGreen | DoBlue;
    int             i;
    float           factor;
    unsigned long   plane;
    unsigned short  intensity;

    dsp = XOpenDisplay(0);
    if ( dsp == 0 )
    {
        *flag_success = FALSE;

        printf("Error : cannot connect to X server %s\n\n",XDisplayName(0));

        return;
    }

    screen = DefaultScreenOfDisplay(dsp);

    root_window = XRootWindowOfScreen(screen);

    *screen_height = HeightOfScreen(screen);
    *screen_width  = WidthOfScreen(screen);

    BLACK = BlackPixelOfScreen(screen);
    WHITE = WhitePixelOfScreen(screen);

    colormap     = DefaultColormapOfScreen(screen);
    depth        = DefaultDepthOfScreen(screen);
    visual       = DefaultVisualOfScreen(screen);
    visual_class = visual->class;

    if ( depth == 1 )
    {
        num_color = 2;
    }
    else
    {
        num_color = MINVAL(CellsOfScreen(screen),MAX_COLORS);
        switch ( visual_class )
        {
            case PseudoColor:

                i = 0;
                while ( (i < num_color)                              &&
                        XAllocColorCells(dsp,colormap,True,
                                         &plane,0,&color_table[i],1)    )
                {
                    screen_color.pixel = color_table[i];
                    screen_color.red   = (unsigned short)
                                         (MAX_INTENSITY * RED[i]);
                    screen_color.green = (unsigned short)
                                         (MAX_INTENSITY * GREEN[i]);
                    screen_color.blue  = (unsigned short)
                                         (MAX_INTENSITY * BLUE[i]);
                    screen_color.flags = flag_modify;

                    XStoreColor(dsp,colormap,&screen_color);

                    i++;
                }
                num_color = i;

                break;

            case DirectColor:
            case StaticColor:
            case TrueColor:

                for ( i = 0 ; i < num_color ; i++ )
                {
                    screen_color.red   = (unsigned short)
                                         (MAX_INTENSITY * RED[i]);
                    screen_color.green = (unsigned short)
                                         (MAX_INTENSITY * GREEN[i]);
                    screen_color.blue  = (unsigned short)
                                         (MAX_INTENSITY * BLUE[i]);
                    screen_color.flags = flag_modify;

                    XAllocColor(dsp,colormap,&screen_color);
                    color_table[i] = screen_color.pixel;
                }

                break;

            case GrayScale:

                i = 2;
                while ( (i < num_color)                              &&
                        XAllocColorCells(dsp,colormap,True,
                                         &plane,0,&color_table[i],1)    )
                {
                    i++;
                }
                num_color = i;

                color_table[0] = BLACK;
                color_table[1] = WHITE;
                for ( i = 2 ; i < num_color ; i++ )
                {
                    factor = (float)(i - 2) / (float)(num_color - 3);
                    intensity = (unsigned short)(MAX_INTENSITY * factor);

                    screen_color.pixel = color_table[i];
                    screen_color.red   = intensity;
                    screen_color.green = intensity;
                    screen_color.blue  = intensity;
                    screen_color.flags = flag_modify;

                    XStoreColor(dsp,colormap,&screen_color);
                }

                break;
        }
    }

    font_struc = XLoadQueryFont(dsp,FONT_REF[0].name);
    if ( font_struc != 0 )
    {
        for ( i = 0 ; i < NUM_FONT ; i++ )
            font[i] = XLoadFont(dsp,FONT_REF[i].name);

        font_id = -1;
    }
    else
    {
        *flag_success = FALSE;

        printf("Error : cannot find the fonts chosen to use\n");
        printf("Please modify the fonts requested in file X11DRV.C.\n");

        return;
    }

    *flag_success = TRUE;

}

/**--------------------------------------------------------------------------**/

x11drw_(x1,y1,x2,y2)
  int  *x1, *y1;
  int  *x2, *y2;
{

    XDrawLine(dsp,plot_window,obj_gc,*x1,*y1,*x2,*y2);

}

/**--------------------------------------------------------------------------**/

x11col_(color_no)
  int *color_no;
{

    XGCValues  gc_value;

    unsigned long  gc_mask;

    gc_mask = GCForeground;

    if ( depth >= 1 )
    {
        gc_value.foreground = color_table[*color_no];
    }
    else
    {
        if ( *color_no == 0 )
            gc_value.foreground = BLACK;
        else
            gc_value.foreground = WHITE;
    }

    XChangeGC(dsp,obj_gc,gc_mask,&gc_value);

}

/**--------------------------------------------------------------------------**/

x11win_(window_x,window_y,window_width,window_height)
  int  *window_x;
  int  *window_y;
  int  *window_width;
  int  *window_height;
{

    XEvent                event;
    XGCValues             gc_value;
    XSetWindowAttributes  window_attr;
    XSizeHints            size_hint;
    XWMHints              wm_hint;

    char           window_name[25];
    int            border_width = 0;
    unsigned long  attr_mask;
    unsigned long  gc_mask;

    if ( plot_window != 0 )
        XDestroyWindow(dsp,plot_window);

    plot_win_height = *window_height;
    plot_win_width  = *window_width;

    if ( depth == 1 )
    {
        plot_window = XCreateSimpleWindow
                      (dsp,root_window,*window_x,*window_y,
                       plot_win_width,plot_win_height,border_width,BLACK,BLACK);
    }
    else
    {
        attr_mask =
        CWBackPixel | CWBorderPixel | CWEventMask | CWOverrideRedirect;
        window_attr.background_pixel  = BLACK;
        window_attr.border_pixel      = BLACK;
        window_attr.event_mask        = ExposureMask | StructureNotifyMask;
        window_attr.override_redirect = False;

        plot_window = XCreateWindow(dsp,root_window,*window_x,*window_y,
                                    plot_win_width,plot_win_height,
                                    border_width,depth,InputOutput,
                                    visual,attr_mask,&window_attr       );
    }

    sprintf(window_name,"  LINGY post-processor window %d",++frame_no);
    XSetIconName(dsp,plot_window,window_name);
    XStoreName(dsp,plot_window,window_name);

    wm_hint.flags = InputHint | StateHint;
    wm_hint.input         = False;
    wm_hint.initial_state = NormalState;
    XSetWMHints(dsp,plot_window,&wm_hint);

    size_hint.flags = USPosition | USSize;
    size_hint.x      = *window_x;
    size_hint.y      = *window_y;
    size_hint.width  = *window_width;
    size_hint.height = *window_height;
    XSetNormalHints(dsp,plot_window,&size_hint);

    gc_mask = GCForeground;
    gc_value.foreground = WHITE;
    obj_gc = XCreateGC(dsp,plot_window,gc_mask,&gc_value);

    XMapWindow(dsp,plot_window);
    while ( TRUE )
    {
        XNextEvent(dsp,&event);
        if ( event.type == Expose )
            break;
    }
    XFlush(dsp);

}

/**--------------------------------------------------------------------------**/

x11fls_()
{

    XFlush(dsp);

}

/**--------------------------------------------------------------------------**/

x11chi_(text_size)
  float  *text_size;
{

    int  i;

    for ( i = 0 ; i < NUM_FONT ; i++ )
        if ( *text_size >= FONT_REF[i].scale )
            break;

    if ( font_id != i )
    {
        font_id = i;
        XSetFont(dsp,obj_gc,font[font_id]);
    }

}

/**--------------------------------------------------------------------------**/

x11chr_(text_x,text_y,text_len,text)
  int   *text_x;
  int   *text_y;
  int   *text_len;
  char  text[];
{

   XDrawString(dsp,plot_window,obj_gc,*text_x,*text_y,text,*text_len);

}

/**--------------------------------------------------------------------------**/

x11fil_(num_vertex,vertex)
  int  *num_vertex;
  int  vertex[][2];
{
    char  *calloc();

    XPoint  *coord;

    int  i;
    int  num_point;

    num_point = *num_vertex + 1;

    coord = (XPoint *)calloc(num_point,sizeof(XPoint));

    for ( i = 0 ; i < *num_vertex ; i++ )
    {
        coord[i].x = (short)(vertex[i][0]);
        coord[i].y = (short)(vertex[i][1]);
    }
    coord[*num_vertex].x = coord[0].x;
    coord[*num_vertex].y = coord[0].y;

    XFillPolygon(dsp,plot_window,obj_gc,
                 coord,*num_vertex,Nonconvex,CoordModeOrigin);
    XDrawLines(dsp,plot_window,obj_gc,coord,num_point,CoordModeOrigin);

    cfree(coord);

}

/**--------------------------------------------------------------------------**/
x11end_()
{

    XDestroyWindow(dsp,plot_window);

    XFlush(dsp);

}
/**--------------------------------------------------------------------------**/
